#include <inttypes.h>
#include <stdio.h>
#include "fmcadc100m14b4cha.h"

#define NSHOT 1
#define PRESAMPLES 100
#define POSTSAMPLES 100


static void adc100m14b4cha_lib_set_up(struct m_suite *m_suite)
{
	struct adc_dev *adc;
	struct adc_conf cfg;
	int err, i;

	err = adc_init();
	m_assert_int_eq(0, err);

	adc = adc_open(ADC100M14B4CHA_TYPE_NAME, adc_devid, 0, 0, ADC_F_FLUSH);
	m_assert_mem_not_null(adc);

	memset(&cfg, 0, sizeof(struct adc_conf));
	cfg.type = ADC_CONF_TYPE_ACQ;
	adc_set_conf(&cfg, ADC_CONF_ACQ_N_SHOTS, NSHOT);
	adc_set_conf(&cfg, ADC_CONF_ACQ_POST_SAMP, POSTSAMPLES);
	adc_set_conf(&cfg, ADC_CONF_ACQ_PRE_SAMP, PRESAMPLES);
	err = adc_apply_config(adc, 0 , &cfg);
	m_assert_int_eq(0, err);

	memset(&cfg, 0, sizeof(struct adc_conf));
	cfg.type = ADC_CONF_TYPE_TRG_EXT;
	adc_set_conf_mask_all(&cfg, adc);
	adc_set_conf(&cfg, ADC_CONF_TRG_EXT_ENABLE, 0);
	err = adc_apply_config(adc, 0 , &cfg);
	m_assert_int_eq(0, err);

	for (i = 0; i < 4; ++i) {
		memset(&cfg, 0, sizeof(struct adc_conf));
		cfg.type = ADC_CONF_TYPE_TRG_THR;
		cfg.route_to = i;
		adc_set_conf_mask_all(&cfg, adc);
		adc_set_conf(&cfg, ADC_CONF_TRG_THR_ENABLE, 0);
		err = adc_apply_config(adc, 0 , &cfg);
		m_assert_int_eq(0, err);

		memset(&cfg, 0, sizeof(struct adc_conf));
		cfg.type = ADC_CONF_TYPE_CHN;
		cfg.route_to = i;
		adc_set_conf(&cfg, ADC_CONF_CHN_SATURATION, 0x7FFF);
		adc_set_conf(&cfg, ADC_CONF_CHN_OFFSET, 0);
		adc_set_conf(&cfg, ADC_CONF_CHN_TERMINATION, 0);
		adc_set_conf(&cfg, ADC_CONF_CHN_RANGE, 0);
		err = adc_apply_config(adc, 0 , &cfg);
		m_assert_int_eq(0, err);
	}

	m_suite->private = adc;

	srandom(time(NULL));
}
static void adc100m14b4cha_lib_tear_down(struct m_suite *m_suite)
{
	struct adc_dev *adc = m_suite->private;

	if (!adc)
		return;
	adc_close(adc);

	adc_exit();
}

#define adc100m14b4cha_lib_trg_test1_desc "It runs an acquisition using the software trigger.\n" \
	"This allows to tests the *poll* (we should get the time-out before the trigger) and the" \
	"*software trigger*"
static void adc100m14b4cha_lib_trg_test1(struct m_test *m_test)
{
	struct adc_dev *adc = m_test->suite->private;
	struct timeval tv = {0, 500 * 1000};
	int ret;

	ret = adc_has_trigger_fire(adc);
	m_assert_int_eq(1, ret);

	/* Any attempt to poll should fail until we do the software trigger */
	ret = adc_acq_start(adc, ADC_F_FLUSH, &tv);
	m_assert_int_eq(-1, ret);
	m_assert_int_eq(EAGAIN, errno);

	tv.tv_usec = random() & 0xFFFFF;
	ret = adc_acq_poll(adc, 0, &tv);
	m_assert_int_eq(-1, ret);
	m_assert_int_eq(EAGAIN, errno);

	/* Now any poll should return with success */
	ret = adc_trigger_fire(adc);
	m_assert_int_eq(0, ret);
	tv.tv_usec = random() & 0xFFFFF;
	ret = adc_acq_poll(adc, 0, &tv);
	m_assert_int_eq(0, ret);

	ret = adc_acq_stop(adc, 0);
	m_assert_int_eq(0, ret);
}


static void adc100m14b4cha_lib_trg_ext_down(struct m_test *m_test)
{
	struct adc_dev *adc = m_test->suite->private;
	struct adc_conf cfg;
	int err;

	err = adc_acq_stop(adc, 0);
	m_assert_int_eq(0, err);

	memset(&cfg, 0, sizeof(struct adc_conf));
	cfg.type = ADC_CONF_TYPE_TRG_EXT;
	adc_set_conf_mask_all(&cfg, adc);
	adc_set_conf(&cfg, ADC_CONF_TRG_EXT_DELAY, 0);
	adc_set_conf(&cfg, ADC_CONF_TRG_EXT_POLARITY, ADC_TRG_POL_NEG);
	adc_set_conf(&cfg, ADC_CONF_TRG_EXT_ENABLE, 0);
	err = adc_apply_config(adc, 0 , &cfg);
	m_assert_int_eq(0, err);
}

static void adc100m14b4cha_lib_trg_test2_up(struct m_test *m_test){}
static void adc100m14b4cha_lib_trg_test2_down(struct m_test *m_test)
{
	adc100m14b4cha_lib_trg_ext_down(m_test);
}

#define adc100m14b4cha_lib_trg_test2_desc "It runs an acquisition using the external trigger\n" \
	"The external trigger pulse frequency must be 1Hz\n" \
	"Here we test that it actually trigger"
static void adc100m14b4cha_lib_trg_test2(struct m_test *m_test)
{
	struct adc_dev *adc = m_test->suite->private;
	struct timeval tv1 = {0, 0};
	struct timeval tv2 = {2, 0}; /* 2.0 s */
	struct adc_conf cfg;
	int err;

	memset(&cfg, 0, sizeof(struct adc_conf));
	cfg.type = ADC_CONF_TYPE_TRG_EXT;
	adc_set_conf_mask_all(&cfg, adc);
	adc_set_conf(&cfg, ADC_CONF_TRG_EXT_DELAY, 0);
	adc_set_conf(&cfg, ADC_CONF_TRG_EXT_POLARITY, ADC_TRG_POL_POS);
	adc_set_conf(&cfg, ADC_CONF_TRG_EXT_ENABLE, 1);
	err = adc_apply_config(adc, 0 , &cfg);
	m_assert_int_eq(0, err);

	err = adc_acq_start(adc, ADC_F_FLUSH, &tv1);
	m_assert_int_eq(0, err);

	/*
	 * wait for the data, it should never never timeout
	 * the external trigger signal is 1Hz
	 */
	err = adc_acq_poll(adc, 0, &tv2);
	m_assert_int_eq(0, err);
}

static void adc100m14b4cha_lib_trg_test3_up(struct m_test *m_test){}
static void adc100m14b4cha_lib_trg_test3_down(struct m_test *m_test)
{
	adc100m14b4cha_lib_trg_ext_down(m_test);
}

#define adc100m14b4cha_lib_trg_test3_desc "It runs an acquisition using the external trigger\n" \
	"The external trigger pulse frequency must be 1Hz\n" \
	"Here we test that the delay parameter."
static void adc100m14b4cha_lib_trg_test3(struct m_test *m_test)
{
	struct adc_dev *adc = m_test->suite->private;
	struct timeval tv1 = {0, 0};
	struct timeval tv2 = {1, 0};
	struct adc_conf cfg;
	uint32_t delay;
	uint64_t usec;
	int err;

	delay = random() & 0x04FF0000; /* do a delay of some ms (less than the
					  external trigger period 1s) */
	memset(&cfg, 0, sizeof(struct adc_conf));
	cfg.type = ADC_CONF_TYPE_TRG_EXT;
	adc_set_conf_mask_all(&cfg, adc);
	adc_set_conf(&cfg, ADC_CONF_TRG_EXT_DELAY, delay);
	adc_set_conf(&cfg, ADC_CONF_TRG_EXT_POLARITY, ADC_TRG_POL_POS);
	adc_set_conf(&cfg, ADC_CONF_TRG_EXT_ENABLE, 1);
	err = adc_apply_config(adc, 0 , &cfg);
	m_assert_int_eq(0, err);

	usec = delay;
	usec = (usec * ADC100M14B4CHA_SAMPLE_PERIOD_NS) / 1000ULL;
	tv2.tv_usec = (long) usec;

	err = adc_acq_start(adc, ADC_F_FLUSH, &tv1);
	m_assert_int_eq(0, err);

	/*
	 * wait for the data, it should never never timeout
	 * the external trigger signal is 1Hz
	 */
	err = adc_acq_poll(adc, 0, &tv2);
	m_assert_int_eq(0, err);
}

static void adc100m14b4cha_lib_trg_test4_up(struct m_test *m_test){}
static void adc100m14b4cha_lib_trg_test4_down(struct m_test *m_test)
{
	adc100m14b4cha_lib_trg_ext_down(m_test);
}

#define adc100m14b4cha_lib_trg_test4_desc "It runs an acquisition using the external trigger\n" \
	"The external trigger pulse frequency must be 1Hz\n" \
	"Here we test that it actually trigger on falling edge"
static void adc100m14b4cha_lib_trg_test4(struct m_test *m_test)
{
	struct adc_dev *adc = m_test->suite->private;
	struct timeval tv1 = {0, 0};
	struct timeval tv2 = {2, 0}; /* 2.0 s */
	struct adc_conf cfg;
	int err;

	memset(&cfg, 0, sizeof(struct adc_conf));
	cfg.type = ADC_CONF_TYPE_TRG_EXT;
	adc_set_conf_mask_all(&cfg, adc);
	adc_set_conf(&cfg, ADC_CONF_TRG_EXT_DELAY, 0);
	adc_set_conf(&cfg, ADC_CONF_TRG_EXT_POLARITY, ADC_TRG_POL_NEG);
	adc_set_conf(&cfg, ADC_CONF_TRG_EXT_ENABLE, 1);
	err = adc_apply_config(adc, 0 , &cfg);
	m_assert_int_eq(0, err);

	err = adc_acq_start(adc, ADC_F_FLUSH, &tv1);
	m_assert_int_eq(0, err);

	/*
	 * wait for the data, it should never never timeout
	 * the external trigger signal is 1Hz
	 */
	err = adc_acq_poll(adc, 0, &tv2);
	m_assert_int_eq(0, err);
}

static void adc100m14b4cha_lib_trg_test5_up(struct m_test *m_test){}
static void adc100m14b4cha_lib_trg_test5_down(struct m_test *m_test)
{
	adc100m14b4cha_lib_trg_ext_down(m_test);
}

#define adc100m14b4cha_lib_trg_test5_desc "It runs an acquisition using the external trigger\n" \
	"The external trigger pulse frequency must be 1Hz\n" \
	"Here we test that the delay parameter works when the delay is greater than" \
	"the period of the external trigger. The ADC should trigger and ignoring any incoming pulse\n"
static void adc100m14b4cha_lib_trg_test5(struct m_test *m_test)
{
	struct adc_dev *adc = m_test->suite->private;
	struct timeval tv1 = {0, 0};
	struct timeval tv2 = {1, 0};
	struct adc_conf cfg;
	uint32_t delay;
	uint64_t usec;
	int err;

	delay = random() & 0xF8000000; /* do a delay of some ms (more than
					the external trigger period */
	memset(&cfg, 0, sizeof(struct adc_conf));
	cfg.type = ADC_CONF_TYPE_TRG_EXT;
	adc_set_conf_mask_all(&cfg, adc);
	adc_set_conf(&cfg, ADC_CONF_TRG_EXT_DELAY, delay);
	adc_set_conf(&cfg, ADC_CONF_TRG_EXT_POLARITY, 0);
	adc_set_conf(&cfg, ADC_CONF_TRG_EXT_ENABLE, 1);
	err = adc_apply_config(adc, 0 , &cfg);
	m_assert_int_eq(0, err);

	usec = delay;
	usec = (usec * ADC100M14B4CHA_SAMPLE_PERIOD_NS) / 1000ULL;
	tv2.tv_usec = (long) usec;

	err = adc_acq_start(adc, ADC_F_FLUSH, &tv1);
	m_assert_int_eq(0, err);

	/*
	 * wait for the data, it should never never timeout
	 * the external trigger signal is 1Hz
	 */
	err = adc_acq_poll(adc, 0, &tv2);
	m_assert_int_eq(0, err);
}



/* * * * THRESHOLD TRIGGER * * * */


static void adc100m14b4cha_lib_trg_thr_down(struct m_test *m_test)
{
	struct adc_dev *adc = m_test->suite->private;
	struct adc_conf cfg;
	int err;
	unsigned long route = (unsigned long)m_test->private;

	err = adc_acq_stop(adc, 0);
	m_assert_int_eq(0, err);

	memset(&cfg, 0, sizeof(struct adc_conf));
	cfg.type = ADC_CONF_TYPE_TRG_THR;
	cfg.route_to = route;
	adc_set_conf_mask_all(&cfg, adc);
	adc_set_conf(&cfg, ADC_CONF_TRG_THR_DELAY, 0);
	adc_set_conf(&cfg, ADC_CONF_TRG_THR_HYSTERESIS, 0);
	adc_set_conf(&cfg, ADC_CONF_TRG_THR_THRESHOLD, 0);
	adc_set_conf(&cfg, ADC_CONF_TRG_THR_POLARITY, ADC_TRG_POL_POS);
	adc_set_conf(&cfg, ADC_CONF_TRG_THR_ENABLE, 0);

	err = adc_apply_config(adc, 0 , &cfg);
	m_assert_int_eq(0, err);
}

static void adc100m14b4cha_lib_trg_thr_pos(struct m_test *m_test, unsigned long channel)
{
	struct adc_dev *adc = m_test->suite->private;
	struct timeval tv1 = {0, 0};
	struct timeval tv2 = {0, 10};
	struct adc_conf cfg;
	uint32_t threshold;
	int err, pattern;

	threshold = random() & 0x0FFF; /* Positive */

	memset(&cfg, 0, sizeof(struct adc_conf));
	cfg.type = ADC_CONF_TYPE_TRG_THR;
	cfg.route_to = channel;
	m_test->private = (void *)channel;
	adc_set_conf_mask_all(&cfg, adc);
	adc_set_conf(&cfg, ADC_CONF_TRG_THR_DELAY, 0);
	adc_set_conf(&cfg, ADC_CONF_TRG_THR_HYSTERESIS, 0);
	adc_set_conf(&cfg, ADC_CONF_TRG_THR_THRESHOLD, threshold);
	adc_set_conf(&cfg, ADC_CONF_TRG_THR_POLARITY, ADC_TRG_POL_POS);
	adc_set_conf(&cfg, ADC_CONF_TRG_THR_ENABLE, 1);
	err = adc_apply_config(adc, 0 , &cfg);
	m_assert_int_eq(0, err);

	/* Start above the threshold */
	pattern = (threshold << 1) >> 2; /* >> 2 because the chip does << 2 */
	err = adc_set_param(adc, "cset0/test-data-pattern",
			    NULL, &pattern);
	m_assert_int_eq(0, err);
	err = adc_acq_start(adc, ADC_F_FLUSH, &tv1);
	m_assert_int_eq(0, err);
	err = adc_acq_poll(adc, 0, &tv2);
	m_assert_int_eq(-1, err);
	m_assert_int_eq(EAGAIN, errno);
	/* Go below the threshold */
	pattern = (threshold >> 1) >> 2; /* >> 2 because the chip does << 2 */
	err = adc_set_param(adc, "cset0/test-data-pattern",
			    NULL, &pattern);
	m_assert_int_eq(0, err);
	err = adc_acq_poll(adc, 0, &tv2);
	m_assert_int_eq(-1, err);
	m_assert_int_eq(EAGAIN, errno);

	/* Go above the threshold */
	pattern = (threshold << 1) >> 2; /* >> 2 because the chip does << 2 */
	err = adc_set_param(adc, "cset0/test-data-pattern",
			    NULL, &pattern);
	m_assert_int_eq(0, err);
	err = adc_acq_poll(adc, 0, &tv2);
	m_assert_int_eq(0, err);
}

static void adc100m14b4cha_lib_trg_thr_neg(struct m_test *m_test, unsigned long channel)
{
	struct adc_dev *adc = m_test->suite->private;
	struct timeval tv1 = {0, 0};
	struct timeval tv2 = {0, 10};
	struct adc_conf cfg;
	uint32_t threshold;
	int err, pattern;

	threshold = random() & 0x0FFF; /* Positive */

	memset(&cfg, 0, sizeof(struct adc_conf));
	cfg.type = ADC_CONF_TYPE_TRG_THR;
	cfg.route_to = channel;
	m_test->private = (void *)channel;
	adc_set_conf_mask_all(&cfg, adc);
	adc_set_conf(&cfg, ADC_CONF_TRG_THR_DELAY, 0);
	adc_set_conf(&cfg, ADC_CONF_TRG_THR_HYSTERESIS, 0);
	adc_set_conf(&cfg, ADC_CONF_TRG_THR_THRESHOLD, threshold);
	adc_set_conf(&cfg, ADC_CONF_TRG_THR_POLARITY, ADC_TRG_POL_NEG);
	adc_set_conf(&cfg, ADC_CONF_TRG_THR_ENABLE, 1);
	err = adc_apply_config(adc, 0 , &cfg);
	m_assert_int_eq(0, err);

	/* Do not trigger */
	pattern = threshold << 1;
	err = adc_set_param(adc, "cset0/test-data-pattern",
			    NULL, &pattern);
	m_assert_int_eq(0, err);
	err = adc_acq_start(adc, ADC_F_FLUSH, &tv1);
	m_assert_int_eq(0, err);
	err = adc_acq_poll(adc, 0, &tv2);
	m_assert_int_eq(-1, err);
	m_assert_int_eq(EAGAIN, errno);
	/* Do trigger */
	pattern = threshold >> 1;
	err = adc_set_param(adc, "cset0/test-data-pattern",
			    NULL, &pattern);
	m_assert_int_eq(0, err);
	err = adc_acq_poll(adc, 0, &tv2);
	m_assert_int_eq(0, err);
}


static void adc100m14b4cha_lib_trg_thr_del(struct m_test *m_test, unsigned long channel)
{
	struct adc_dev *adc = m_test->suite->private;
	struct timeval tv1 = {0, 0};
	struct timeval tv2 = {1, 0};
	struct adc_conf cfg;
	uint32_t delay;
	uint32_t threshold;
	int err, pattern;

	threshold = random() & 0x0FFF; /* Positive */
	delay = random();

	memset(&cfg, 0, sizeof(struct adc_conf));
	cfg.type = ADC_CONF_TYPE_TRG_THR;
	cfg.route_to = channel;
	m_test->private = (void *)channel;
	adc_set_conf_mask_all(&cfg, adc);
	adc_set_conf(&cfg, ADC_CONF_TRG_THR_DELAY, delay);
	adc_set_conf(&cfg, ADC_CONF_TRG_THR_HYSTERESIS, 0);
	adc_set_conf(&cfg, ADC_CONF_TRG_THR_THRESHOLD, threshold);
	adc_set_conf(&cfg, ADC_CONF_TRG_THR_POLARITY, ADC_TRG_POL_POS);
	adc_set_conf(&cfg, ADC_CONF_TRG_THR_ENABLE, 1);
	err = adc_apply_config(adc, 0 , &cfg);
	m_assert_int_eq(0, err);

	/* Do not trigger */
	pattern = threshold << 1;
	err = adc_set_param(adc, "cset0/test-data-pattern",
			    NULL, &pattern);
	m_assert_int_eq(0, err);
	err = adc_acq_start(adc, ADC_F_FLUSH, &tv1);
	m_assert_int_eq(0, err);
	err = adc_acq_poll(adc, 0, &tv2);
	m_assert_int_eq(-1, err);
	/* Do trigger */
	pattern = threshold >> 1;
	err = adc_set_param(adc, "cset0/test-data-pattern",
			    NULL, &pattern);
	m_assert_int_eq(0, err);
	err = adc_acq_poll(adc, 0, &tv2);
	m_assert_int_eq(0, err);
}

#define adc100m14b4cha_lib_trg_test6_desc \
	"It runs an acquisition using the threshold trigger on channel 1\n" \
	"Positive slope. Pattern data"
static void adc100m14b4cha_lib_trg_test6_up(struct m_test *m_test){}
static void adc100m14b4cha_lib_trg_test6_down(struct m_test *m_test)
{
	adc100m14b4cha_lib_trg_thr_down(m_test);
}
static void adc100m14b4cha_lib_trg_test6(struct m_test *m_test)
{
	adc100m14b4cha_lib_trg_thr_pos(m_test, 0);
}


#define adc100m14b4cha_lib_trg_test7_desc \
	"It runs an acquisition using the threshold trigger on channel 2\n" \
	"Positive slope. Pattern data"
static void adc100m14b4cha_lib_trg_test7_up(struct m_test *m_test){}
static void adc100m14b4cha_lib_trg_test7_down(struct m_test *m_test)
{
	adc100m14b4cha_lib_trg_thr_down(m_test);
}
static void adc100m14b4cha_lib_trg_test7(struct m_test *m_test)
{
	adc100m14b4cha_lib_trg_thr_pos(m_test, 1);
}

#define adc100m14b4cha_lib_trg_test8_desc \
	"It runs an acquisition using the threshold trigger on channel 3\n" \
	"Positive slope. Pattern data"
static void adc100m14b4cha_lib_trg_test8_up(struct m_test *m_test){}
static void adc100m14b4cha_lib_trg_test8_down(struct m_test *m_test)
{
	adc100m14b4cha_lib_trg_thr_down(m_test);
}
static void adc100m14b4cha_lib_trg_test8(struct m_test *m_test)
{
	adc100m14b4cha_lib_trg_thr_pos(m_test, 2);
}

#define adc100m14b4cha_lib_trg_test9_desc \
	"It runs an acquisition using the threshold trigger on channel 4\n" \
	"Positive slope. Pattern data"
static void adc100m14b4cha_lib_trg_test9_up(struct m_test *m_test){}
static void adc100m14b4cha_lib_trg_test9_down(struct m_test *m_test)
{
	adc100m14b4cha_lib_trg_thr_down(m_test);
}
static void adc100m14b4cha_lib_trg_test9(struct m_test *m_test)
{
	adc100m14b4cha_lib_trg_thr_pos(m_test, 3);
}


#define adc100m14b4cha_lib_trg_test10_desc \
	"It runs an acquisition using the threshold trigger on channel 1\n" \
	"Negative slope. Pattern data"
static void adc100m14b4cha_lib_trg_test10_up(struct m_test *m_test){}
static void adc100m14b4cha_lib_trg_test10_down(struct m_test *m_test)
{
	adc100m14b4cha_lib_trg_thr_down(m_test);
}
static void adc100m14b4cha_lib_trg_test10(struct m_test *m_test)
{
	adc100m14b4cha_lib_trg_thr_neg(m_test, 0);
}


#define adc100m14b4cha_lib_trg_test11_desc \
	"It runs an acquisition using the threshold trigger on channel 2\n" \
	"Negative slope. Pattern data"
static void adc100m14b4cha_lib_trg_test11_up(struct m_test *m_test){}
static void adc100m14b4cha_lib_trg_test11_down(struct m_test *m_test)
{
	adc100m14b4cha_lib_trg_thr_down(m_test);
}
static void adc100m14b4cha_lib_trg_test11(struct m_test *m_test)
{
	adc100m14b4cha_lib_trg_thr_neg(m_test, 1);
}

#define adc100m14b4cha_lib_trg_test12_desc \
	"It runs an acquisition using the threshold trigger on channel 3\n" \
	"Negative slope. Pattern data"
static void adc100m14b4cha_lib_trg_test12_up(struct m_test *m_test){}
static void adc100m14b4cha_lib_trg_test12_down(struct m_test *m_test)
{
	adc100m14b4cha_lib_trg_thr_down(m_test);
}
static void adc100m14b4cha_lib_trg_test12(struct m_test *m_test)
{
	adc100m14b4cha_lib_trg_thr_neg(m_test, 2);
}

#define adc100m14b4cha_lib_trg_test13_desc \
	"It runs an acquisition using the threshold trigger on channel 4\n" \
	"Negative slope. Pattern data"
static void adc100m14b4cha_lib_trg_test13_up(struct m_test *m_test){}
static void adc100m14b4cha_lib_trg_test13_down(struct m_test *m_test)
{
	adc100m14b4cha_lib_trg_thr_down(m_test);
}
static void adc100m14b4cha_lib_trg_test13(struct m_test *m_test)
{
	adc100m14b4cha_lib_trg_thr_neg(m_test, 3);
}


#define adc100m14b4cha_lib_trg_test14_desc \
	"It runs an acquisition using the threshold trigger on channel 4\n" \
	"Positive slope. Pattern data. Random delay."
static void adc100m14b4cha_lib_trg_test14_up(struct m_test *m_test){}
static void adc100m14b4cha_lib_trg_test14_down(struct m_test *m_test)
{
	adc100m14b4cha_lib_trg_thr_down(m_test);
}
static void adc100m14b4cha_lib_trg_test14(struct m_test *m_test)
{
	adc100m14b4cha_lib_trg_thr_del(m_test, 0);
}


#define adc100m14b4cha_lib_trg_m_test_ud(_n) m_test_desc_loop(adc100m14b4cha_lib_trg_test ## _n ## _up, \
							      adc100m14b4cha_lib_trg_test ## _n, \
							      adc100m14b4cha_lib_trg_test ## _n ## _down,	\
							      adc100m14b4cha_lib_trg_test ##_n ## _desc, \
							      LOOP)
#define adc100m14b4cha_lib_trg_m_test(_n) m_test_desc_loop(NULL, \
							   adc100m14b4cha_lib_trg_test ## _n, \
							   NULL, \
							   adc100m14b4cha_lib_trg_test ##_n ## _desc, \
							   LOOP)
static struct m_test adc100m14b4cha_lib_tests[] = {
	adc100m14b4cha_lib_trg_m_test(1),
	adc100m14b4cha_lib_trg_m_test_ud(2),
	adc100m14b4cha_lib_trg_m_test_ud(3),
	adc100m14b4cha_lib_trg_m_test_ud(4),
	adc100m14b4cha_lib_trg_m_test_ud(5),
	adc100m14b4cha_lib_trg_m_test_ud(6),
	adc100m14b4cha_lib_trg_m_test_ud(7),
	adc100m14b4cha_lib_trg_m_test_ud(8),
	adc100m14b4cha_lib_trg_m_test_ud(9),
	adc100m14b4cha_lib_trg_m_test_ud(10),
	adc100m14b4cha_lib_trg_m_test_ud(11),
	adc100m14b4cha_lib_trg_m_test_ud(12),
	adc100m14b4cha_lib_trg_m_test_ud(13),
	adc100m14b4cha_lib_trg_m_test_ud(14),
	/* adc100m14b4cha_lib_trg_m_test_ud(15), */
	/* adc100m14b4cha_lib_trg_m_test_ud(16), */
	/* adc100m14b4cha_lib_trg_m_test_ud(17), */
};

struct m_suite adc100m14b4cha_lib_trg_suite = {
	.name = "FMC ADC 100M 4Bit 4Channel",
	.desc = "this collection of tests validate some expected behaviour",
	.flags = M_VERBOSE | M_ERRNO_CHECK | M_ERRNO_FUNC,
	.private = NULL,
	.tests = adc100m14b4cha_lib_tests,
	.test_count = M_ARRAY_SIZE(adc100m14b4cha_lib_tests),
	.set_up = adc100m14b4cha_lib_set_up,
	.tear_down = adc100m14b4cha_lib_tear_down,
	.strerror = NULL,
};
